/* arch/i386/kernel/dynamic_syscall.c
 *	Entry code for dynamic syscalls on i386.
 */
#include <linux/kernel.h>
#include <linux/compiler.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <asm/ptrace.h>
#include <asm/uaccess.h>

struct dummy_args {
	long data[8];
};

extern struct vsyscall_entry {
	long eip;
	long (*call)(struct dummy_args args);
} vsyscall_list_begin, vsyscall_list_end;

long sys_dynamic_syscall(struct pt_regs regs) __attribute__((regparm(0)));

long sys_dynamic_syscall(struct pt_regs regs)
{
	struct dummy_args dummy_args;
	struct vsyscall_entry *ent = (void *)regs.edx;
	void *args = (void *)regs.ecx;
	long ret;

	pr_debug("ent = %p  args = %p\n", ent, args);
	pr_debug("eip = 0x%08lx\n", regs.eip);

	if (unlikely(!current->mm->vsys_mapped))
		goto err;

	/* The pointer must be aligned in the table. */
	if (unlikely((long)ent & (sizeof(*ent) - 1))) {
		pr_debug("unaligned\n");
		goto err;
	}

	/* Bounds checking... */
	if (unlikely(ent < &vsyscall_list_begin) ||
	    unlikely(ent >= &vsyscall_list_end)) {
		pr_debug("out of range %p <= %p < %p\n", 
			&vsyscall_list_begin, ent,
			&vsyscall_list_end);
		goto err;
	}
	/* The entry should be valid now.  Verify that the caller's eip 
	 * is correct.
	 */
	if (unlikely(ent->eip != regs.eip)) {
		pr_debug("eip mismatch (0x%lx vs 0x%lx)\n", ent->eip, regs.eip);
		goto err;
	}

	pr_debug("ent->call = %p\n", ent->call);

	if (unlikely(verify_area(VERIFY_READ, args, sizeof(dummy_args))))
		return -EFAULT;
	
	__asm__ volatile (
	"	cld			\n"
	"	sub $0x20, %%esp		\n"
	"	movl %%esp, %%edi		\n"
	"	movl $0x8, %%ecx		\n"
	"1:	rep movsl		\n"
	"	call %%edx		\n"
	"2:	add $0x20, %%esp		\n"

/* the exception handling: just return -EFAULT */
	".section .fixup, \"ax\"	\n"
	"3:	movl $0xfffffff2, %%eax	\n"  /* -EFAULT */
	"	jmp 2b			\n"
	".previous			\n"
	".section __ex_table,\"a\"	\n"
	"       .align 4		\n"
	"       .long 1b, 3b		\n"
	".previous			\n"

	: "=a" (ret)
	: "S" (args), "d" (ent->call)
	: "%edi", "%ecx" );

	pr_debug("ret = 0x%08lx\n", ret);

	return ret;
err:
	return -ENOSYS;
}
