#ifndef __LINUX__AIO_H
#define __LINUX__AIO_H

#include <linux/tqueue.h>
#include <linux/kiovec.h>
#include <linux/list.h>
#include <asm/atomic.h>

#define AIO_MAXSEGS		4
#define AIO_KIOGRP_NR_ATOMIC	8

struct kioctx;

struct kiocb {
	struct list_head	list;
	int		(*cancel)(struct kioctx *ctx, struct kiocb *kiocb);
	struct file	*filp;
	struct kioctx	*ctx;
	void		*user_obj;
	__u64		user_data;
	loff_t		pos;
	unsigned long	buf;
	size_t		nr_transferred;	/* used for chunking */
	size_t		size;
	size_t		this_size;
	unsigned	key;		/* id of this request */
	struct tq_struct	tq;	/* argh. */
};

#define AIO_RING_PAGES	8
#define AIO_RING_PAGES	8
struct kioctx {
	atomic_t		users;
	int			dead;
	struct mm_struct	*mm;

	/* This needs improving */
	unsigned long		user_id;
	struct kioctx		*next;

	wait_queue_head_t	wait;

	spinlock_t		lock;

	int			reqs_active;
	struct list_head	free_reqs;
	struct list_head	active_reqs;	/* used for cancellation */

	unsigned		max_reqs;
	unsigned		ring_mask;
	struct aio_ring		*ring;
	spinlock_t		ring_lock;
	int			ring_was_vmallocd;
	int			ring_order;
};

extern struct file_operations aio_fops;

extern void FASTCALL(aio_complete(struct kiocb *iocb, long res, long res2));
extern void __put_ioctx(struct kioctx *ctx);
struct mm_struct;
extern void exit_aio(struct mm_struct *mm);

#define get_ioctx(kioctx)	do { if (unlikely(atomic_read(&(kioctx)->users) <= 0)) BUG(); atomic_inc(&(kioctx)->users); } while (0)
#define put_ioctx(kioctx)	do { if (unlikely(atomic_dec_and_test(&(kioctx)->users))) __put_ioctx(kioctx); else if (unlikely(atomic_read(&(kioctx)->users) < 0)) BUG(); } while (0)

#include <linux/aio_abi.h>

static inline struct kiocb *list_kiocb(struct list_head *h)
{
	return list_entry(h, struct kiocb, list);
}

struct file;
extern int generic_aio_poll(struct file *file, struct kiocb *req, struct iocb iocb);
extern ssize_t generic_aio_read(struct file *file, struct kiocb *req, struct iocb iocb, size_t min_size);
extern ssize_t generic_aio_write(struct file *file, struct kiocb *req, struct iocb iocb, size_t min_size);
extern ssize_t generic_file_aio_read(struct file *file, struct kiocb *req, struct iocb iocb);
extern ssize_t generic_file_aio_write(struct file *file, struct kiocb *req, struct iocb iocb);
extern ssize_t generic_sock_aio_read(struct file *file, struct kiocb *req, struct iocb iocb);

/* for sysctl: */
extern unsigned aio_nr, aio_max_nr, aio_max_size, aio_max_pinned;

#endif /* __LINUX__AIO_H */
